@@ -1,4 +1,4 @@
-/* $Id: VBoxNetFltInternal.h 86926 2013-07-03 11:07:54Z aleksey $ */
+/* $Id: VBoxNetFltInternal.h 101501 2015-07-07 18:55:05Z vushakov $ */
 /** @file
  * VBoxNetFlt - Network Filter Driver (Host), Internal Header.
  */
@@ -153,6 +153,8 @@ typedef struct VBOXNETFLTINS
             bool volatile fSetPromiscuous;
             /** The MAC address of the interface. */
             RTMAC MacAddr;
+            /** PF_SYSTEM socket to listen for events (XXX: globals?) */
+            socket_t pSysSock;
             /** @} */
 # elif defined(RT_OS_LINUX)
             /** @name Linux instance data
@@ -170,7 +172,9 @@ typedef struct VBOXNETFLTINS
             bool volatile fPacketHandler;
             /** The MAC address of the interface. */
             RTMAC MacAddr;
-            struct notifier_block Notifier;
+            struct notifier_block Notifier; /* netdevice */
+            struct notifier_block NotifierIPv4;
+            struct notifier_block NotifierIPv6;
             struct packet_type    PacketType;
 #  ifndef VBOXNETFLT_LINUX_NO_XMIT_QUEUE
             struct sk_buff_head   XmitQueue;
