@@ -1,4 +1,4 @@
-/* $Id: VBoxNetAdp-linux.c 83576 2013-02-04 15:54:15Z fmehnert $ */
+/* $Id: VBoxNetAdp-linux.c 102794 2015-09-21 15:41:19Z fmehnert $ */
 /** @file
  * VBoxNetAdp - Virtual Network Adapter Driver (Host), Linux Specific Code.
  */
@@ -183,6 +183,9 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
     /* No need for private data. */
     pNetDev = alloc_netdev(sizeof(VBOXNETADPPRIV),
                            pThis->szName[0] ? pThis->szName : VBOXNETADP_LINUX_NAME,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
+                           NET_NAME_UNKNOWN,
+#endif
                            vboxNetAdpNetDevInit);
     if (pNetDev)
     {
@@ -420,7 +423,6 @@ static int __init VBoxNetAdpLinuxInit(void)
  */
 static void __exit VBoxNetAdpLinuxUnload(void)
 {
-    int rc;
     Log(("VBoxNetAdpLinuxUnload\n"));
 
     /*
@@ -429,11 +431,7 @@ static void __exit VBoxNetAdpLinuxUnload(void)
 
     vboxNetAdpShutdown();
     /* Remove control device */
-    rc = misc_deregister(&g_CtlDev);
-    if (rc < 0)
-    {
-        printk(KERN_ERR "misc_deregister failed with rc=%x\n", rc);
-    }
+    misc_deregister(&g_CtlDev);
 
     RTR0Term();
 
