@@ -1,4 +1,4 @@
-/* $Id: VBoxPci-linux.c 84701 2013-04-03 09:48:39Z fmehnert $ */
+/* $Id: VBoxPci-linux.c 102452 2015-09-02 09:48:29Z fmehnert $ */
 /** @file
  * VBoxPci - PCI Driver (Host), Linux Specific Code.
  */
@@ -136,7 +136,9 @@ static int __init VBoxPciLinuxInit(void)
     {
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
         /* find_module() is static before Linux 2.6.30 */
+        mutex_lock(&module_mutex);
         g_VBoxPciGlobals.pciStubModule = find_module(PCI_STUB_MODULE_NAME);
+        mutex_unlock(&module_mutex);
         if (g_VBoxPciGlobals.pciStubModule)
         {
             if (try_module_get(g_VBoxPciGlobals.pciStubModule))
@@ -885,7 +887,8 @@ int vboxPciOsDevRegisterIrqHandler(PVBOXRAWPCIINS pIns, PFNRAWPCIISR pfnHandler,
 #else
 
                      /* We don't allow interrupts sharing */
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+                     /* XXX overhaul */
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20) && LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 0)
                      IRQF_DISABLED, /* keep irqs disabled when calling the action handler */
 # else
                      0,
