@@ -1,4 +1,4 @@
-/* $Id: RTLogWriteDebugger-r0drv-linux.c 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: RTLogWriteDebugger-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Log To Debugger, Ring-0 Driver, Linux.
  */
@@ -35,7 +35,9 @@
 
 RTDECL(void) RTLogWriteDebugger(const char *pch, size_t cb)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     printk("%.*s", (int)cb, pch);
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 RT_EXPORT_SYMBOL(RTLogWriteDebugger);
 
