@@ -1,4 +1,4 @@
-/* $Id: alloc-r0drv-linux.c 95611 2014-08-20 10:36:04Z fmehnert $ */
+/* $Id: alloc-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Memory Allocation, Ring-0 Driver, Linux.
  */
@@ -164,7 +164,7 @@ static PRTMEMHDR rtR0MemAllocExecVmArea(size_t cb)
     pVmArea->nr_pages = 0;    /* paranoia? */
     pVmArea->pages    = NULL; /* paranoia? */
 
-    papPages = (struct page **)kmalloc(cPages * sizeof(papPages[0]), GFP_KERNEL);
+    papPages = (struct page **)kmalloc(cPages * sizeof(papPages[0]), GFP_KERNEL | __GFP_NOWARN);
     if (!papPages)
     {
         vunmap(pVmArea->addr);
@@ -227,6 +227,7 @@ static PRTMEMHDR rtR0MemAllocExecVmArea(size_t cb)
 DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
 {
     PRTMEMHDR pHdr;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     /*
      * Allocate.
@@ -254,11 +255,11 @@ DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
 
 # else  /* !RTMEMALLOC_EXEC_HEAP */
 # error "you don not want to go here..."
-        pHdr = (PRTMEMHDR)__vmalloc(cb + sizeof(*pHdr), GFP_KERNEL | __GFP_HIGHMEM, MY_PAGE_KERNEL_EXEC);
+        pHdr = (PRTMEMHDR)__vmalloc(cb + sizeof(*pHdr), GFP_KERNEL | __GFP_HIGHMEM | __GFP_NOWARN, MY_PAGE_KERNEL_EXEC);
 # endif /* !RTMEMALLOC_EXEC_HEAP */
 
 #elif defined(PAGE_KERNEL_EXEC) && defined(CONFIG_X86_PAE)
-        pHdr = (PRTMEMHDR)__vmalloc(cb + sizeof(*pHdr), GFP_KERNEL | __GFP_HIGHMEM, MY_PAGE_KERNEL_EXEC);
+        pHdr = (PRTMEMHDR)__vmalloc(cb + sizeof(*pHdr), GFP_KERNEL | __GFP_HIGHMEM | __GFP_NOWARN, MY_PAGE_KERNEL_EXEC);
 #else
         pHdr = (PRTMEMHDR)vmalloc(cb + sizeof(*pHdr));
 #endif
@@ -276,7 +277,8 @@ DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
         {
             fFlags |= RTMEMHDR_FLAG_KMALLOC;
             pHdr = kmalloc(cb + sizeof(*pHdr),
-                           (fFlags & RTMEMHDR_FLAG_ANY_CTX_ALLOC) ? GFP_ATOMIC : GFP_KERNEL);
+                           (fFlags & RTMEMHDR_FLAG_ANY_CTX_ALLOC) ? (GFP_ATOMIC | __GFP_NOWARN)
+                                                                  : (GFP_KERNEL | __GFP_NOWARN));
             if (RT_UNLIKELY(   !pHdr
                             && cb > PAGE_SIZE
                             && !(fFlags & RTMEMHDR_FLAG_ANY_CTX) ))
@@ -289,7 +291,10 @@ DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
             pHdr = vmalloc(cb + sizeof(*pHdr));
     }
     if (RT_UNLIKELY(!pHdr))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VERR_NO_MEMORY;
+    }
 
     /*
      * Initialize.
@@ -300,6 +305,7 @@ DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
     pHdr->cbReq     = cb;
 
     *ppHdr = pHdr;
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 
@@ -309,6 +315,8 @@ DECLHIDDEN(int) rtR0MemAllocEx(size_t cb, uint32_t fFlags, PRTMEMHDR *ppHdr)
  */
 DECLHIDDEN(void) rtR0MemFree(PRTMEMHDR pHdr)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
     pHdr->u32Magic += 1;
     if (pHdr->fFlags & RTMEMHDR_FLAG_KMALLOC)
         kfree(pHdr);
@@ -337,6 +345,8 @@ DECLHIDDEN(void) rtR0MemFree(PRTMEMHDR pHdr)
 #endif
     else
         vfree(pHdr);
+
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 
@@ -375,6 +385,8 @@ RTR0DECL(void *) RTMemContAlloc(PRTCCPHYS pPhys, size_t cb)
     int             cOrder;
     unsigned        cPages;
     struct page    *paPages;
+    void           *pvRet;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     /*
      * validate input.
@@ -426,10 +438,13 @@ RTR0DECL(void *) RTMemContAlloc(PRTCCPHYS pPhys, size_t cb)
 #endif
         }
         *pPhys = page_to_phys(paPages);
-        return phys_to_virt(page_to_phys(paPages));
+        pvRet = phys_to_virt(page_to_phys(paPages));
     }
+    else
+        pvRet = NULL;
 
-    return NULL;
+    IPRT_LINUX_RESTORE_EFL_AC();
+    return pvRet;
 }
 RT_EXPORT_SYMBOL(RTMemContAlloc);
 
@@ -448,6 +463,7 @@ RTR0DECL(void) RTMemContFree(void *pv, size_t cb)
         unsigned        cPages;
         unsigned        iPage;
         struct page    *paPages;
+        IPRT_LINUX_SAVE_EFL_AC();
 
         /* validate */
         AssertMsg(!((uintptr_t)pv & PAGE_OFFSET_MASK), ("pv=%p\n", pv));
@@ -470,6 +486,7 @@ RTR0DECL(void) RTMemContFree(void *pv, size_t cb)
 #endif
         }
         __free_pages(paPages, cOrder);
+        IPRT_LINUX_RESTORE_EFL_AC();
     }
 }
 RT_EXPORT_SYMBOL(RTMemContFree);
