@@ -1,4 +1,4 @@
-/* $Id: assert-r0drv-linux.c 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: assert-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT -  Assertion Workers, Ring-0 Drivers, Linux.
  */
@@ -42,23 +42,27 @@
 
 DECLHIDDEN(void) rtR0AssertNativeMsg1(const char *pszExpr, unsigned uLine, const char *pszFile, const char *pszFunction)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     printk(KERN_EMERG
            "\r\n!!Assertion Failed!!\r\n"
            "Expression: %s\r\n"
            "Location  : %s(%d) %s\r\n",
            pszExpr, pszFile, uLine, pszFunction);
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 
 DECLHIDDEN(void) rtR0AssertNativeMsg2V(bool fInitial, const char *pszFormat, va_list va)
 {
     char szMsg[256];
+    IPRT_LINUX_SAVE_EFL_AC();
 
     RTStrPrintfV(szMsg, sizeof(szMsg) - 1, pszFormat, va);
     szMsg[sizeof(szMsg) - 1] = '\0';
     printk(KERN_EMERG "%s", szMsg);
 
     NOREF(fInitial);
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 
