@@ -1,4 +1,4 @@
-/* $Id: initterm-r0drv-linux.c 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: initterm-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Initialization & Termination, R0 Driver, Linux.
  */
@@ -64,6 +64,8 @@ DECLHIDDEN(void) rtR0MemExecCleanup(void);
  */
 DECLHIDDEN(void) rtR0LnxWorkqueuePush(RTR0LNXWORKQUEUEITEM *pWork, void (*pfnWorker)(RTR0LNXWORKQUEUEITEM *))
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 41)
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
     INIT_WORK(pWork, pfnWorker);
@@ -75,6 +77,8 @@ DECLHIDDEN(void) rtR0LnxWorkqueuePush(RTR0LNXWORKQUEUEITEM *pWork, void (*pfnWor
     INIT_TQUEUE(pWork, (void (*)(void *))pfnWorker, pWork);
     queue_task(pWork, &g_rtR0LnxWorkQueue);
 #endif
+
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 
@@ -86,28 +90,38 @@ DECLHIDDEN(void) rtR0LnxWorkqueuePush(RTR0LNXWORKQUEUEITEM *pWork, void (*pfnWor
  */
 DECLHIDDEN(void) rtR0LnxWorkqueueFlush(void)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 41)
     flush_workqueue(g_prtR0LnxWorkQueue);
 #else
     run_task_queue(&g_rtR0LnxWorkQueue);
 #endif
+
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 
 DECLHIDDEN(int) rtR0InitNative(void)
 {
+    int rc = VINF_SUCCESS;
+    IPRT_LINUX_SAVE_EFL_AC();
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 41)
     g_prtR0LnxWorkQueue = create_workqueue("iprt");
     if (!g_prtR0LnxWorkQueue)
-        return VERR_NO_MEMORY;
+        rc = VERR_NO_MEMORY;
 #endif
 
-    return VINF_SUCCESS;
+    IPRT_LINUX_RESTORE_EFL_AC();
+    return rc;
 }
 
 
 DECLHIDDEN(void) rtR0TermNative(void)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
     rtR0LnxWorkqueueFlush();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 41)
     destroy_workqueue(g_prtR0LnxWorkQueue);
@@ -117,5 +131,7 @@ DECLHIDDEN(void) rtR0TermNative(void)
 #if defined(RT_ARCH_AMD64) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
     rtR0MemExecCleanup();
 #endif
+
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
