@@ -1,4 +1,4 @@
-/* $Id: memuserkernel-r0drv-linux.c 83576 2013-02-04 15:54:15Z fmehnert $ */
+/* $Id: memuserkernel-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - User & Kernel Memory, Ring-0 Driver, Linux.
  */
@@ -37,8 +37,13 @@
 
 RTR0DECL(int) RTR0MemUserCopyFrom(void *pvDst, RTR3PTR R3PtrSrc, size_t cb)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     if (RT_LIKELY(copy_from_user(pvDst, (void *)R3PtrSrc, cb) == 0))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VINF_SUCCESS;
+    }
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VERR_ACCESS_DENIED;
 }
 RT_EXPORT_SYMBOL(RTR0MemUserCopyFrom);
@@ -46,8 +51,13 @@ RT_EXPORT_SYMBOL(RTR0MemUserCopyFrom);
 
 RTR0DECL(int) RTR0MemUserCopyTo(RTR3PTR R3PtrDst, void const *pvSrc, size_t cb)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     if (RT_LIKELY(copy_to_user((void *)R3PtrDst, pvSrc, cb) == 0))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VINF_SUCCESS;
+    }
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VERR_ACCESS_DENIED;
 }
 RT_EXPORT_SYMBOL(RTR0MemUserCopyTo);
@@ -55,7 +65,10 @@ RT_EXPORT_SYMBOL(RTR0MemUserCopyTo);
 
 RTR0DECL(bool) RTR0MemUserIsValidAddr(RTR3PTR R3Ptr)
 {
-    return access_ok(VERIFY_READ, (void *)R3Ptr, 1);
+    IPRT_LINUX_SAVE_EFL_AC();
+    bool fRc = access_ok(VERIFY_READ, (void *)R3Ptr, 1);
+    IPRT_LINUX_RESTORE_EFL_AC();
+    return fRc;
 }
 RT_EXPORT_SYMBOL(RTR0MemUserIsValidAddr);
 
@@ -63,25 +76,10 @@ RT_EXPORT_SYMBOL(RTR0MemUserIsValidAddr);
 RTR0DECL(bool) RTR0MemKernelIsValidAddr(void *pv)
 {
     /* Couldn't find a straight forward way of doing this... */
-#ifdef RT_ARCH_X86
-# ifdef CONFIG_X86_HIGH_ENTRY
+#if defined(RT_ARCH_X86) && defined(CONFIG_X86_HIGH_ENTRY)
     return true; /* ?? */
-# else
+#elif defined(RT_ARCH_X86) || defined(RT_ARCH_AMD64)
     return (uintptr_t)pv >= PAGE_OFFSET;
-# endif
-
-#elif defined(RT_ARCH_AMD64)
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 11)
-    /* Linux 2.6.0 ... 2.6.10 set PAGE_OFFSET to 0x0000010000000000.
-     * Linux 2.6.11 sets PAGE_OFFSET to 0xffff810000000000.
-     * Linux 2.6.33 sets PAGE_OFFSET to 0xffff880000000000. */
-    return (uintptr_t)pv >= PAGE_OFFSET;
-# else
-    /* Not correct (KERNEL_TEXT_START=0xffffffff80000000),
-     * VMALLOC_START (0xffffff0000000000) would be better */
-    return (uintptr_t)pv >= KERNEL_TEXT_START;
-# endif
-
 #else
 # error "PORT ME"
     return !access_ok(VERIFY_READ, pv, 1);
@@ -128,6 +126,7 @@ static int rtR0MemKernelCopyLnxWorker(void *pvDst, void const *pvSrc, size_t cb)
 #  endif
 # endif /* !_ASM_EXTABLE */
     int rc;
+    IPRT_LINUX_SAVE_EFL_AC(); /* paranoia */
     if (!cb)
         return VINF_SUCCESS;
 
@@ -150,6 +149,7 @@ static int rtR0MemKernelCopyLnxWorker(void *pvDst, void const *pvSrc, size_t cb)
                             "2" (pvSrc),
                             "3" (cb)
                           : "memory");
+    IPRT_LINUX_RESTORE_EFL_AC();
     return rc;
 #else
     return VERR_NOT_SUPPORTED;
