@@ -1,4 +1,4 @@
-/* $Id: mpnotification-r0drv-linux.c 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: mpnotification-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Multiprocessor Event Notifications, Ring-0 Driver, Linux.
  */
@@ -199,12 +199,14 @@ static int rtMpNotificationLinuxCallback(struct notifier_block *pNotifierBlock,
 DECLHIDDEN(int) rtR0MpNotificationNativeInit(void)
 {
     int rc;
+    IPRT_LINUX_SAVE_EFL_AC();
 
 # ifdef CPU_DOWN_FAILED
     RTCpuSetEmpty(&g_MpPendingOfflineSet);
 # endif
 
     rc = register_cpu_notifier(&g_NotifierBlock);
+    IPRT_LINUX_RESTORE_EFL_AC();
     AssertMsgReturn(!rc, ("%d\n", rc), RTErrConvertFromErrno(rc));
     return VINF_SUCCESS;
 }
@@ -212,7 +214,9 @@ DECLHIDDEN(int) rtR0MpNotificationNativeInit(void)
 
 DECLHIDDEN(void) rtR0MpNotificationNativeTerm(void)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     unregister_cpu_notifier(&g_NotifierBlock);
+    IPRT_LINUX_RESTORE_EFL_AC();
 }
 
 #else   /* Not supported / Not needed */
