@@ -1,4 +1,4 @@
-/* $Id: semeventmulti-r0drv-linux.c 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: semeventmulti-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Multiple Release Event Semaphores, Ring-0 Driver, Linux.
  */
@@ -95,6 +95,7 @@ RTDECL(int)  RTSemEventMultiCreateEx(PRTSEMEVENTMULTI phEventMultiSem, uint32_t
                                      const char *pszNameFmt, ...)
 {
     PRTSEMEVENTMULTIINTERNAL pThis;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     AssertReturn(!(fFlags & ~RTSEMEVENTMULTI_FLAGS_NO_LOCK_VAL), VERR_INVALID_PARAMETER);
     pThis = (PRTSEMEVENTMULTIINTERNAL)RTMemAlloc(sizeof(*pThis));
@@ -106,8 +107,10 @@ RTDECL(int)  RTSemEventMultiCreateEx(PRTSEMEVENTMULTI phEventMultiSem, uint32_t
         init_waitqueue_head(&pThis->Head);
 
         *phEventMultiSem = pThis;
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VINF_SUCCESS;
     }
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VERR_NO_MEMORY;
 }
 RT_EXPORT_SYMBOL(RTSemEventMultiCreate);
@@ -143,6 +146,8 @@ DECLINLINE(void) rtR0SemEventMultiLnxRelease(PRTSEMEVENTMULTIINTERNAL pThis)
 
 RTDECL(int) RTSemEventMultiDestroy(RTSEMEVENTMULTI hEventMultiSem)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
     /*
      * Validate input.
      */
@@ -161,6 +166,8 @@ RTDECL(int) RTSemEventMultiDestroy(RTSEMEVENTMULTI hEventMultiSem)
     Assert(!waitqueue_active(&pThis->Head));
     wake_up_all(&pThis->Head);
     rtR0SemEventMultiLnxRelease(pThis);
+
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTSemEventMultiDestroy);
@@ -168,6 +175,7 @@ RT_EXPORT_SYMBOL(RTSemEventMultiDestroy);
 
 RTDECL(int) RTSemEventMultiSignal(RTSEMEVENTMULTI hEventMultiSem)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
     uint32_t fNew;
     uint32_t fOld;
 
@@ -197,6 +205,7 @@ RTDECL(int) RTSemEventMultiSignal(RTSEMEVENTMULTI hEventMultiSem)
     wake_up_all(&pThis->Head);
 
     rtR0SemEventMultiLnxRelease(pThis);
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTSemEventMultiSignal);
@@ -260,6 +269,7 @@ static int rtR0SemEventMultiLnxWait(PRTSEMEVENTMULTIINTERNAL pThis, uint32_t fFl
          * We have to wait.
          */
         RTR0SEMLNXWAIT Wait;
+        IPRT_LINUX_SAVE_EFL_AC();
         rc = rtR0SemLnxWaitInit(&Wait, fFlags, uTimeout, &pThis->Head);
         if (RT_SUCCESS(rc))
         {
@@ -295,6 +305,7 @@ static int rtR0SemEventMultiLnxWait(PRTSEMEVENTMULTIINTERNAL pThis, uint32_t fFl
             rtR0SemLnxWaitDelete(&Wait);
             IPRT_DEBUG_SEMS_STATE_RC(pThis, 'E', rc);
         }
+        IPRT_LINUX_RESTORE_EFL_AC();
     }
 
     rtR0SemEventMultiLnxRelease(pThis);
