@@ -1,4 +1,4 @@
-/* $Id: time-r0drv-linux.c 83575 2013-02-04 14:27:54Z fmehnert $ */
+/* $Id: time-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Time, Ring-0 Driver, Linux.
  */
@@ -169,14 +169,17 @@ RT_EXPORT_SYMBOL(RTTimeSystemMilliTS);
 
 RTDECL(PRTTIMESPEC) RTTimeNow(PRTTIMESPEC pTime)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
     struct timespec Ts;
     ktime_get_real_ts(&Ts);
+    IPRT_LINUX_RESTORE_EFL_AC();
     return RTTimeSpecSetTimespec(pTime, &Ts);
 
 #else   /* < 2.6.16 */
     struct timeval Tv;
     do_gettimeofday(&Tv);
+    IPRT_LINUX_RESTORE_EFL_AC();
     return RTTimeSpecSetTimeval(pTime, &Tv);
 #endif
 }
