@@ -1,4 +1,4 @@
-/* $Id: timer-r0drv-linux.c 83575 2013-02-04 14:27:54Z fmehnert $ */
+/* $Id: timer-r0drv-linux.c 103354 2015-10-13 14:24:51Z bird $ */
 /** @file
  * IPRT - Timers, Ring-0 Driver, Linux.
  */
@@ -1185,6 +1185,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
 {
     RTTIMERLINUXSTARTONCPUARGS Args;
     int rc2;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     /*
      * Validate.
@@ -1202,7 +1203,11 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
      * Omni timer?
      */
     if (pTimer->fAllCpus)
-        return rtTimerLnxOmniStart(pTimer, &Args);
+    {
+        rc2 = rtTimerLnxOmniStart(pTimer, &Args);
+        IPRT_LINUX_RESTORE_EFL_AC();
+        return rc2;
+    }
 #endif
 
     /*
@@ -1233,6 +1238,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
                             return rc2;
                         }
                     }
+                    IPRT_LINUX_RESTORE_EFL_AC();
                     return VINF_SUCCESS;
                 }
                 break;
@@ -1242,12 +1248,14 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
                 if (rtTimerLnxCmpXchgState(&pTimer->aSubTimers[0].enmState, RTTIMERLNXSTATE_CB_RESTARTING, enmState))
                 {
                     ASMAtomicWriteBool(&pTimer->fSuspended, false);
+                    IPRT_LINUX_RESTORE_EFL_AC();
                     return VINF_SUCCESS;
                 }
                 break;
 
             default:
                 AssertMsgFailed(("%d\n", enmState));
+                IPRT_LINUX_RESTORE_EFL_AC();
                 return VERR_INTERNAL_ERROR_4;
         }
         ASMNopPause();
@@ -1328,6 +1336,7 @@ RTDECL(int) RTTimerStop(PRTTIMER pTimer)
     /*
      * Validate.
      */
+    IPRT_LINUX_SAVE_EFL_AC();
     AssertPtrReturn(pTimer, VERR_INVALID_HANDLE);
     AssertReturn(pTimer->u32Magic == RTTIMER_MAGIC, VERR_INVALID_HANDLE);
     RTTIMERLNX_LOG(("stop %p\n", pTimer));
@@ -1336,6 +1345,8 @@ RTDECL(int) RTTimerStop(PRTTIMER pTimer)
         return VERR_TIMER_SUSPENDED;
 
     rtTimerLnxStop(pTimer, false /*fForDestroy*/);
+
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTTimerStop);
@@ -1345,6 +1356,7 @@ RTDECL(int) RTTimerChangeInterval(PRTTIMER pTimer, uint64_t u64NanoInterval)
 {
     unsigned long cJiffies;
     unsigned long flFlags;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     /*
      * Validate.
@@ -1362,6 +1374,7 @@ RTDECL(int) RTTimerChangeInterval(PRTTIMER pTimer, uint64_t u64NanoInterval)
     if (pTimer->fHighRes)
     {
         ASMAtomicWriteU64(&pTimer->u64NanoInterval, u64NanoInterval);
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VINF_SUCCESS;
     }
 #endif
@@ -1382,6 +1395,7 @@ RTDECL(int) RTTimerChangeInterval(PRTTIMER pTimer, uint64_t u64NanoInterval)
     pTimer->cJiffies = cJiffies;
     ASMAtomicWriteU64(&pTimer->u64NanoInterval, u64NanoInterval);
     spin_unlock_irqrestore(&pTimer->ChgIntLock, flFlags);
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTTimerChangeInterval);
@@ -1390,6 +1404,7 @@ RT_EXPORT_SYMBOL(RTTimerChangeInterval);
 RTDECL(int) RTTimerDestroy(PRTTIMER pTimer)
 {
     bool fCanDestroy;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     /*
      * Validate. It's ok to pass NULL pointer.
@@ -1457,6 +1472,7 @@ RTDECL(int) RTTimerDestroy(PRTTIMER pTimer)
             rtTimerLnxDestroyIt(pTimer);
     }
 
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTTimerDestroy);
@@ -1468,6 +1484,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
     RTCPUID     iCpu;
     unsigned    cCpus;
     int         rc;
+    IPRT_LINUX_SAVE_EFL_AC();
 
     rtR0LnxWorkqueueFlush();                /* for 2.4 */
     *ppTimer = NULL;
@@ -1476,11 +1493,17 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
      * Validate flags.
      */
     if (!RTTIMER_FLAGS_ARE_VALID(fFlags))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VERR_INVALID_PARAMETER;
+    }
     if (    (fFlags & RTTIMER_FLAGS_CPU_SPECIFIC)
         &&  (fFlags & RTTIMER_FLAGS_CPU_ALL) != RTTIMER_FLAGS_CPU_ALL
         &&  !RTMpIsCpuPossible(RTMpCpuIdFromSetIndex(fFlags & RTTIMER_FLAGS_CPU_MASK)))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VERR_CPU_NOT_FOUND;
+    }
 
     /*
      * Allocate the timer handler.
@@ -1491,14 +1514,17 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
     {
         cCpus = RTMpGetMaxCpuId() + 1;
         Assert(cCpus <= RTCPUSET_MAX_CPUS); /* On linux we have a 1:1 relationship between cpuid and set index. */
-        AssertReturn(u64NanoInterval, VERR_NOT_IMPLEMENTED); /* We don't implement single shot on all cpus, sorry. */
+        AssertReturnStmt(u64NanoInterval, IPRT_LINUX_RESTORE_EFL_AC(), VERR_NOT_IMPLEMENTED); /* We don't implement single shot on all cpus, sorry. */
     }
 #endif
 
     rc = RTMemAllocEx(RT_OFFSETOF(RTTIMER, aSubTimers[cCpus]), 0,
                       RTMEMALLOCEX_FLAGS_ZEROED | RTMEMALLOCEX_FLAGS_ANY_CTX_FREE, (void **)&pTimer);
     if (RT_FAILURE(rc))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return rc;
+    }
 
     /*
      * Initialize it.
@@ -1565,6 +1591,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
         if (RT_FAILURE(rc))
         {
             RTTimerDestroy(pTimer);
+            IPRT_LINUX_RESTORE_EFL_AC();
             return rc;
         }
     }
@@ -1572,6 +1599,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
 
     RTTIMERLNX_LOG(("create %p hires=%d fFlags=%#x cCpus=%u\n", pTimer, pTimer->fHighRes, fFlags, cCpus));
     *ppTimer = pTimer;
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTTimerCreateEx);
@@ -1582,7 +1610,10 @@ RTDECL(uint32_t) RTTimerGetSystemGranularity(void)
 #if 0 /** @todo Not sure if this is what we want or not... Add new API for
        *        querying the resolution of the high res timers? */
     struct timespec Ts;
-    int rc = hrtimer_get_res(CLOCK_MONOTONIC, &Ts);
+    int rc;
+    IPRT_LINUX_SAVE_EFL_AC();
+    rc = hrtimer_get_res(CLOCK_MONOTONIC, &Ts);
+    IPRT_LINUX_RESTORE_EFL_AC();
     if (!rc)
     {
         Assert(!Ts.tv_sec);
